/*******************************************************************************

    uBlock Origin - a browser extension to block requests.
    Copyright (C) 2019-present Raymond Hill

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see {http://www.gnu.org/licenses/}.

    Home: https://github.com/gorhill/uBlock
*/

/* global CodeMirror */

'use strict';

CodeMirror.defineMode('ubo-dynamic-filtering', ( ) => {

    return {
        token: stream => {
            stream.skipToEnd();
            return null;
        }
    };
});

CodeMirror.registerHelper('fold', 'ubo-dynamic-filtering', (cm, start) => {
    function isHeader(lineNo) {
        const tokentype = cm.getTokenTypeAt(CodeMirror.Pos(lineNo, 0));
        return tokentype && /\bheader\b/.test(tokentype);
    }

    function headerLevel(lineNo, line, nextLine) {
        let match = line && line.match(/^#+/);
        if (match && isHeader(lineNo)) return match[0].length;
        match = nextLine && nextLine.match(/^[=\-]+\s*$/);
        if (match && isHeader(lineNo + 1)) return nextLine[0] === '=' ? 1 : 2;
        return 100;
    }

    const firstLine = cm.getLine(start.line);
    let nextLine = cm.getLine(start.line + 1);
    const level = headerLevel(start.line, firstLine, nextLine);
    if ( level === 100 ) { return; }

    const lastLineNo = cm.lastLine();
    let end = start.line,
        nextNextLine = cm.getLine(end + 2);
    while ( end < lastLineNo ) {
        if ( headerLevel(end + 1, nextLine, nextNextLine) <= level ) { break; }
        ++end;
        nextLine = nextNextLine;
        nextNextLine = cm.getLine(end + 2);
    }

    return {
        from: CodeMirror.Pos(start.line, firstLine.length),
        to: CodeMirror.Pos(end, cm.getLine(end).length),
    };
});
